﻿using System;
using System.Diagnostics;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Markup;
using log4net;
using Triton.Bot;
using Triton.Bot.Settings;
using Triton.Common;
using Triton.Game;
using Logger = Triton.Common.LogUtilities.Logger;
using System.Text;


//namespace HREngine.Bots
namespace ToolBox
{
    /*This plug-in:
    1) allows you to save misplay (button - 'I see misplay');
    2) add gameMode info to the UILogg;
    3) add win/loss/concede info to the UILogg;
    4) add Rank info to the UILogg;
    5) assigns an unique number to each of yours Battle.net accounts and adds this number to the battle log (Log and UILogg).
    On the basis of this unique number you can divide game statistics for different accounts.
    No one except you can know yours Battle.net info even if he knows your secret word.
    */

    public class ToolBox : IPlugin
    {
        private string mySecretWord = "";
        private string gameMode = "";
        private string HeroLabel = "";
        private string accName = "";
        private string temp = Environment.CurrentDirectory + @"\HSBToolBox\temp";

        private static readonly ILog Log = Logger.GetLoggerInstanceForType();

        private bool _enabled;

        #region Implementation of IPlugin

        /// <summary> The name of the plugin. </summary>
        public string Name
        {
            get { return "ToolBox"; }
        }

        /// <summary> The description of the plugin. </summary>
        public string Description
        {
            get { return "This plug-in::\r\n- allows you to save misplay (button - 'I see misplay');\r\n- add gameMode info to the UILogg;\r\n- add Rank info to the UILogg;\r\n- add win/loss/concede info to the UILogg;\r\n- add a unique number to separate the statistics by accounts."; }
        }

        /// <summary>The author of the plugin.</summary>
        public string Author
        {
            get { return "Lazy_cat"; }
        }

        /// <summary>The version of the plugin.</summary>
        public string Version
        {
            get { return "0.0.1.3"; }
        }

        /// <summary>Initializes this object. This is called when the object is loaded into the bot.</summary>
        public void Initialize()
        {
            Log.DebugFormat("[ToolBox] Initialize");
        }

        /// <summary> The plugin start callback. Do any initialization here. </summary>
        public void Start()
        {
            Log.DebugFormat("[ToolBox] Start");
            GameEventManager.GameOver += GameEventManagerOnGameOver;
            GameEventManager.CustomDecksCached += GameEventManagerCustomDecksCached;
        }

        /// <summary> The plugin tick callback. Do any update logic here. </summary>
        public void Tick()
        {
        }

        /// <summary> The plugin stop callback. Do any pre-dispose cleanup here. </summary>
        public void Stop()
        {
            Log.DebugFormat("[ToolBox] Stop");
            GameEventManager.GameOver -= GameEventManagerOnGameOver;
            GameEventManager.CustomDecksCached -= GameEventManagerCustomDecksCached;
        }

        public JsonSettings Settings
        {
            get { return ToolBoxSettings.Instance; }
        }

        private UserControl _control;

        /// <summary> The plugin's settings control. This will be added to the Hearthbuddy Settings tab.</summary>
        public UserControl Control
        {
            get
            {
                if (_control != null)
                {
                    return _control;
                }

                using (var fs = new FileStream(@"Plugins\ToolBox\SettingsGui.xaml", FileMode.Open))
                {
                    var root = (UserControl)XamlReader.Load(fs);

                    // Your settings binding here.
					
                    // MySecretWord
                    if (!Wpf.SetupTextBoxBinding(root, "MySecretWordTextBox", "MySecretWord",
                        BindingMode.TwoWay, ToolBoxSettings.Instance))
                    {
                        Log.DebugFormat("[SettingsControl] SetupTextBoxBinding failed for 'MySecretWordTextBox'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }

                    // writeRank
                    if (!Wpf.SetupCheckBoxBinding(root, "writeRankChb", "writeRank",
                        BindingMode.TwoWay, ToolBoxSettings.Instance))
                    {
                        Log.DebugFormat("[SettingsControl] SetupCheckBoxBinding failed for 'writeRank'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }

                    // writeCardsList
                    if (!Wpf.SetupCheckBoxBinding(root, "writeCardsListChb", "writeCardsList",
                        BindingMode.TwoWay, ToolBoxSettings.Instance))
                    {
                        Log.DebugFormat("[SettingsControl] SetupCheckBoxBinding failed for 'writeCardsList'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }
					
                    // Your settings event handlers here.

                    var IseeMisplayButtonL = Wpf.FindControlByName<Button>(root, "IseeMisplayButtonL");
                    IseeMisplayButtonL.Click += IseeMisplayButtonOnClick;
                    var IseeMisplayButtonR = Wpf.FindControlByName<Button>(root, "IseeMisplayButtonR");
                    IseeMisplayButtonR.Click += IseeMisplayButtonOnClick;

                    _control = root;
                }

                return _control;
            }
        }

        /// <summary>Is this plugin currently enabled?</summary>
        public bool IsEnabled
        {
            get { return _enabled; }
        }

        /// <summary> The plugin is being enabled.</summary>
        public void Enable()
        {
            Log.DebugFormat("[ToolBox] Enable");
            getHeroLabel();
            _enabled = true;
        }

        /// <summary> The plugin is being disabled.</summary>
        public void Disable()
        {
            Log.DebugFormat("[ToolBox] Disable");
            _enabled = false;
        }

        #endregion

        #region Implementation of IDisposable

        /// <summary>Deinitializes this object. This is called when the object is being unloaded from the bot.</summary>
        public void Deinitialize()
        {
        }

        #endregion

        #region Override of Object

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return Name + ": " + Description;
        }

        #endregion

        
        private string sha1(string input)
        {
            byte[] hash;
            using (var sha1 = new System.Security.Cryptography.SHA1CryptoServiceProvider())
                hash = sha1.ComputeHash(Encoding.Unicode.GetBytes(input));
            var sb = new StringBuilder();
            foreach (byte b in hash) sb.AppendFormat("{0:x2}", b);
            return sb.ToString();
        }

        private void getHeroLabel()
        {
            if (HeroLabel == "" || mySecretWord != ToolBoxSettings.Instance.MySecretWord)
            {
                mySecretWord = ToolBoxSettings.Instance.MySecretWord;
                if (Triton.Game.Mapping.RankMgr.Get() != null)
                {
                    var myPlayer = Triton.Game.Mapping.BnetPresenceMgr.Get().GetMyPlayer();
                    if (myPlayer != null)
                    {
                        accName = myPlayer.GetBestName();
                        if (mySecretWord == "" || mySecretWord == "changeItToYourSecretWord!")
                        {
                            mySecretWord = sha1(Directory.GetParent(@"Plugins").CreationTime.ToString());
                            ToolBoxSettings.Instance.MySecretWord = mySecretWord;
                        }
                        HeroLabel = sha1(sha1(myPlayer.m_account.m_battleTag.m_name + myPlayer.m_account.m_battleTag.m_number + mySecretWord));
                        int sLen = 9;
                        int parts = HeroLabel.Length / sLen + 1;
                        ulong number = 0;
                        for (int i = 0; i < parts; i++)
                        {
                            int startPos = i * sLen;
                            if ((i + 1) * sLen > HeroLabel.Length) sLen = HeroLabel.Length - i * sLen;
                            string s = HeroLabel.Substring(startPos, sLen);
                            try
                            {
                                number += Convert.ToUInt64(s, 16);
                            }
                            catch (Exception e) { }
                        }
                        HeroLabel = number.ToString();
                        Log.InfoFormat("[ToolBox] HeroLabel: " + HeroLabel);
                    }
                }
            }
        }

        private void GameEventManagerOnGameOver(object sender, GameOverEventArgs gameOverEventArgs)
        {
            getHeroLabel();
            string winORloss = "Win";
            if (gameOverEventArgs.Result == GameOverFlag.Defeat)
            {
                if (gameOverEventArgs.Conceded) winORloss = "Concede";
                else winORloss = "Loss";
            }
            string gameMode = Triton.Bot.Logic.Bots.DefaultBot.DefaultBotSettings.Instance.GameMode.ToString();
            HREngine.Bots.Helpfunctions.Instance.logg("");
            HREngine.Bots.Helpfunctions.Instance.logg("[ToolBox] " + HeroLabel + " " + gameMode + " " + winORloss + (ToolBoxSettings.Instance.writeRank ? " Rank: " + TritonHs.MyRank : ""));
            string gold = Triton.Game.Mapping.NetCache.Get().GetGoldBalance().ToString();
            File.WriteAllText(temp, HeroLabel + " " + gold + " " + TritonHs.MyRank + " " + winORloss);
        }

        private void GameEventManagerCustomDecksCached(object sender, CustomDecksCachedEventArgs CustomDecksCachedEventArgs)
        {
            if (!ToolBoxSettings.Instance.writeCardsList) return;
            StringBuilder allCards = new StringBuilder("", 10000);
            allCards.Append("HeroLabel:").Append(HeroLabel).Append("\r\n");
            allCards.Append("Dust:").Append(Triton.Game.Mapping.NetCache.Get().GetArcaneDustBalance()).Append("\r\n");
            string gold = Triton.Game.Mapping.NetCache.Get().GetGoldBalance().ToString();
            allCards.Append("Gold:").Append(gold).Append("\r\n");
            var myCollection = Triton.Game.Mapping.CollectionManager.Get().GetOwnedCards();
            foreach (var onecard in myCollection)
            {
                allCards.Append(onecard.CardId).Append(":").Append(onecard.OwnedCount).Append(":").Append(onecard.PremiumType == Triton.Game.Mapping.TAG_PREMIUM.GOLDEN ? 1 : 0);
                allCards.Append(":").Append(onecard.IsCraftable ? 1 : 0).Append(":").Append(onecard.IsNewCard ? 1 : 0).Append("\r\n");
            }
            
            try
            {
                string pathToCardsList = Environment.CurrentDirectory + @"\HSBToolBox\CardsList";
                string pathToCard = pathToCardsList + @"\" + HeroLabel + ".cardslist";
                if (!Directory.Exists(pathToCardsList)) Directory.CreateDirectory(pathToCardsList);
                File.WriteAllText(pathToCard, allCards.ToString());

                pathToCardsList = Environment.CurrentDirectory + @"\cardimg\herolabel";
                pathToCard = pathToCardsList + @"\" + HeroLabel + ".hlbl";
                if (!Directory.Exists(pathToCardsList)) Directory.CreateDirectory(pathToCardsList);
                File.WriteAllText(pathToCard, accName);

                File.WriteAllText(temp, HeroLabel + " " + gold + " " + TritonHs.MyRank);
                Log.InfoFormat("[ToolBox] Cards list saved successfully.");
            }
            catch { }
        }

        private void IseeMisplayButtonOnClick(object sender, RoutedEventArgs routedEventArgs)
        {
            var logfile = HREngine.Bots.Settings.Instance.logfile;
            var logpath = HREngine.Bots.Settings.Instance.logpath;
            string turnStep = HREngine.Bots.Hrtprozis.Instance.gTurn.ToString() + "/" + HREngine.Bots.Hrtprozis.Instance.gTurnStep.ToString();
            try
            {
                Microsoft.Win32.SaveFileDialog saveFileDlg = new Microsoft.Win32.SaveFileDialog();
                saveFileDlg.Title = "Save misplay";
                saveFileDlg.FileName = "describe the problem";
                saveFileDlg.Filter = "Text documents (.txt)|*.txt";

                Nullable<bool> dlgResult = saveFileDlg.ShowDialog();
                if (dlgResult == true)
                {
                    var parentLogFullPath = logpath + System.IO.Path.DirectorySeparatorChar + logfile;
                    File.Copy(parentLogFullPath, saveFileDlg.FileName, true);
                    using (StreamWriter sw = File.AppendText(saveFileDlg.FileName))
                    {
                        sw.WriteLine("\r\n[ToolBox] Parent log: " + turnStep + " " + logfile);
                    }
                    Log.DebugFormat("[ToolBox] UILogg saved successfully: " + saveFileDlg.FileName);
                    Log.DebugFormat("[ToolBox] Open this file in the HSBToolBox and describe the problem.");
                }
            }
            catch (Exception ex)
            {
                Log.DebugFormat("*-*");
                Log.DebugFormat("[ToolBox] Save Error, try again! " + ex.Message);
                Log.DebugFormat(":(");
            }
        }

    }
}
