﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using log4net;
using Newtonsoft.Json;
using Triton.Bot.Settings;
using Triton.Common;
using Triton.Game.Mapping;
using Logger = Triton.Common.LogUtilities.Logger;

namespace ToolBox
{
    /// <summary>Settings for the ToolBox plugin. </summary>
    public class ToolBoxSettings : JsonSettings
    {
        private static readonly ILog Log = Logger.GetLoggerInstanceForType();

        private static ToolBoxSettings _instance;

        /// <summary>The current instance for this class. </summary>
        public static ToolBoxSettings Instance
        {
            get { return _instance ?? (_instance = new ToolBoxSettings()); }
        }

        /// <summary>The default ctor. Will use the settings path "ToolBox".</summary>
        public ToolBoxSettings()
            : base(GetSettingsFilePath(Configuration.Instance.Name, string.Format("{0}.json", "ToolBox")))
        {
        }
		
		 private string _mySecretWord;
         private bool _writeRank;
         private bool _writeCardsList;

        /// <summary>The secret word to use.</summary>
        [DefaultValue("changeIt!")]
        public string MySecretWord
        {
            get { return _mySecretWord; }
            set
            {
                if (value.Equals(_mySecretWord)) return;
                _mySecretWord = value;
                NotifyPropertyChanged(() => MySecretWord);
            }
        }

        /// <summary>
        /// Should the plugin to write current rank to the UILogg?
        /// </summary>
        [DefaultValue(true)]
        public bool writeRank
        {
            get { return _writeRank; }
            set
            {
                if (value.Equals(_writeRank)) return;
                _writeRank = value;
                NotifyPropertyChanged(() => writeRank);
                Log.InfoFormat("[ToolBoxSettings] Write rank = {0}.", _writeRank);
            }
        }
        
        /// <summary>
        /// Should the plugin to write acc name to the CardList file?
        /// </summary>
        [DefaultValue(true)]
        public bool writeCardsList
        {
            get { return _writeCardsList; }
            set
            {
                if (value.Equals(_writeCardsList)) return;
                _writeCardsList = value;
                NotifyPropertyChanged(() => writeCardsList);
                Log.InfoFormat("[ToolBoxSettings] Write rank = {0}.", _writeCardsList);
            }
        }
    }
}
